from __future__ import annotations

import re
from enum import Enum
from typing import Optional

import adapter.adapter_server as adapter
from command_line_tool.utils import JsonBlob


class PromptType(str, Enum):
    CONST = "const"
    BOOL = "bool"
    LIST = "list"
    ASSERT = "assert"
    FOR_EACH = "for_each"


class Instruction:
    def __init__(
        self,
        name: str,
        number: str,
        definition: JsonBlob,
        code: str,
        files: dict[str, str],
        debug_mode: bool = False,
        debug_folder_path: Optional[str] = None,
        instruction_parts: Optional[list[str]] = None,
    ) -> None:
        self.name = name
        self.number = number
        self.definition = definition
        self.code = code
        self.files = files
        self.debug_mode = debug_mode
        self.debug_folder_path = debug_folder_path
        self.current_queries: list[tuple[adapter.GPTRole, str]] = []
        self.prompts = list(range(len(self.definition["prompts"])))
        self.attribute_names = []
        self.current_prompts: list[int] = []

        if instruction_parts:
            for i, instruction_part in enumerate(instruction_parts):
                instruction_parts[i] = instruction_part.strip()
            if len(instruction_parts) - 1 != len(self.definition["inputs"]):
                raise ValueError(
                    f"Invalid number of arguments to instruction {number[:-1]}:"
                    f" expected {len(self.definition['inputs'])} but got"
                    f" {len(instruction_parts) - 1}!"
                )

            for i, var_name in enumerate(self.definition["inputs"]):
                setattr(self, var_name, instruction_parts[i + 1])
                self.attribute_names.append(var_name)

    def done(self) -> bool:
        return not self.prompts

    def get_variables_used(self, prompt: JsonBlob) -> list[str]:
        variables_used = []
        if (
            prompt["type"] == PromptType.CONST
            or prompt["type"] == PromptType.BOOL
            or prompt["type"] == PromptType.LIST
        ):
            variables_used = re.findall(r"{(\w+)}", prompt["string"]) + prompt.get("files", [])
        elif prompt["type"] == PromptType.ASSERT:
            variables_used = re.findall(r"{(\w+)}", prompt["condition"]) + re.findall(
                r"{(\w+)}", prompt["feedback"]
            )
        else:
            raise ValueError(f"Unknown prompt type: {prompt['type']}")
        return variables_used

    def check_prompt_ready(self, prompt: JsonBlob) -> bool:
        variables_used = self.get_variables_used(prompt)
        for variable in variables_used:
            if not hasattr(self, variable):
                return False
        return True

    def get_ready_prompts(self) -> list[int]:
        ready_prompts = []
        for prompt_number in self.prompts:
            if self.check_prompt_ready(self.definition["prompts"][prompt_number]):
                ready_prompts.append(prompt_number)
        for prompt_number in ready_prompts:
            self.prompts.remove(prompt_number)
        return ready_prompts

    def generate_query(self, prompt: JsonBlob) -> tuple[adapter.GPTRole, str]:
        variables_used = self.get_variables_used(prompt)
        var_dict = {var: getattr(self, var) for var in variables_used}
        query = prompt["string"].format(**var_dict)
        if prompt.get("files"):
            code_in_files = True
            for file_name in prompt["files"]:
                if not self.files.get(getattr(self, file_name)):
                    print(f"Warning: file not found {file_name}! Defaulting to all code.")
                    code = self.code
                    code_in_files = False
                    break
            if code_in_files:
                code = "\n\n\n\n".join(
                    [self.files[getattr(self, file_name)] for file_name in prompt["files"]]
                )
        else:
            code = self.code
        full_query = f"In the following code, {query}\n\nCode:\n{code}"
        return (adapter.GPTRole.USER, full_query)

    def check_assertion(self, prompt: JsonBlob) -> Optional[str]:
        variables_used = self.get_variables_used(prompt)
        var_dict = {var: getattr(self, var) for var in variables_used}
        if not eval(prompt["condition"].format(**var_dict)):
            assert isinstance(prompt["feedback"], str)
            return prompt["feedback"].format(**var_dict)
        return None

    def process_prompts(self) -> tuple[list[tuple[adapter.GPTRole, str]], list[str]]:
        ret_feedback = []
        ready_prompts = self.get_ready_prompts()
        self.current_prompts = []
        if not ready_prompts:
            print(self.prompts)
            print(self.get_ready_prompts())
            raise RuntimeError("Error: Bad instruction definition! Cannot proceed.")
        queries = []
        for prompt_number in ready_prompts:
            prompt = self.definition["prompts"][prompt_number]
            if (
                prompt["type"] == PromptType.CONST
                or prompt["type"] == PromptType.BOOL
                or prompt["type"] == PromptType.LIST
            ):
                queries.append(self.generate_query(prompt))
                self.current_prompts.append(prompt_number)
            elif prompt["type"] == PromptType.ASSERT:
                feedback = self.check_assertion(prompt)
                if feedback:
                    ret_feedback.append(f"{self.number} {feedback}")
            elif prompt["type"] == PromptType.FOR_EACH:
                raise NotImplementedError("'For each' not yet implemented")
            else:
                raise ValueError(f"Unknown prompt type: {prompt['type']}")
        if self.debug_mode:
            self.current_queries = queries
        return queries, ret_feedback

    def process_single_feedback(self, prompt: JsonBlob, feedback: str) -> None:
        match prompt["type"]:
            case PromptType.CONST:
                setattr(self, prompt["returns"], feedback)
                self.attribute_names.append(prompt["returns"])
            case PromptType.BOOL:
                setattr(self, prompt["returns"], "yes" in feedback.lower())
                self.attribute_names.append(prompt["returns"])
            case PromptType.LIST:
                setattr(self, prompt["returns"], [item.strip() for item in feedback.split(",")])
                self.attribute_names.append(prompt["returns"])
            case _:
                raise ValueError(f"Error: Unknown prompt type \"{prompt['type']}\"")

    def process_feedback(self, all_feedback: list[str]) -> None:
        for i, feedback in enumerate(all_feedback):
            if self.debug_mode:
                print(
                    f"Prompt: {self.definition['prompts'][self.current_prompts[i]]}\nQuery:"
                    f" {self.current_queries[i][1][:self.current_queries[i][1].find('Code:')]}"
                    f"\nFeedback: {feedback}\n"
                )
            self.process_single_feedback(
                self.definition["prompts"][self.current_prompts[i]], feedback
            )

    def get_sub_instructions(self) -> list[Instruction]:
        sub_instructions = []
        sub_instruction_defs = self.definition["sub_instructions"]
        for sub_instruction_def in sub_instruction_defs:
            for value in getattr(self, sub_instruction_def["list"]):
                instruction = Instruction(
                    sub_instruction_def["name"],
                    self.number,
                    sub_instruction_def,
                    self.code,
                    self.files,
                    debug_mode=self.debug_mode,
                    debug_folder_path=self.debug_folder_path,
                )
                setattr(instruction, sub_instruction_def["variable"], value)
                instruction.attribute_names.append(sub_instruction_def["variable"])
                for attribute_name in self.attribute_names:
                    setattr(instruction, attribute_name, getattr(self, attribute_name))
                    instruction.attribute_names.append(attribute_name)
                sub_instructions.append(instruction)
        return sub_instructions



import sys
import os

from marker.command_line_tool.utils import write_json_to_file, write_to_file
from marker.marker.interpreter import Interpreter

def main(mark_scheme_path: (str | None)) -> None:
    # Check for correct number of arguments.
    if len(sys.argv) != 2:
        raise ValueError(
            f"Insufficient arguments! Expected 2 arguments but got {len(sys.argv) - 1}"
        )

    # Mark code and write feedback.
    interpreter = Interpreter(mark_scheme_path, sys.argv[1], debug_mode=True)
    interpreter.run()

    marks = interpreter.get_marks()
    feedback = interpreter.get_feedback()

    write_to_file("./comments.txt", "\n\n".join(feedback))
    write_json_to_file("./marks.json", marks)

    interpreter.print_warnings()

if __name__ == "__main__":
    main(os.getenv('EXERCISE_NAME'))



import json
import re
from typing import Optional

import adapter.adapter_server as adapter
from command_line_tool.instruction import Instruction, PromptType
from command_line_tool.utils import (
    JsonBlob,
    get_file_contents,
    get_folder_contents,
    get_folder_files,
    write_to_file,
)


class DSL:
    def __init__(
        self,
        instructions_defs_path: str,
        instructions_path: str,
        code_path: str,
        debug_folder_path: Optional[str] = None,
        debug_mode: bool = False,
    ) -> None:
        instruction_defs_files = get_folder_files(instructions_defs_path)
        self.instruction_defs: JsonBlob = {}
        for file_contents in instruction_defs_files:
            self.instruction_defs |= self.parse_dsl(file_contents)

        self.debug_folder_path = debug_folder_path
        self.debug_mode = debug_mode
        self.gpt_adapter = adapter.ServerAdapter()
        self.query_index = 1

        if code_path[-1] == "\\":
            code_path = code_path[:-1]
        self.files = get_folder_contents(code_path)
        self.code = "\n\n\n\n".join(self.files.values())

        self.parse_instructions(get_file_contents(instructions_path))

    def print_defs(self) -> None:
        print(json.dumps(self.instruction_defs, indent=4))

    def get_indent(self, line: str) -> int:
        num = 0
        for char in line:
            if char.isspace():
                num += 1
            else:
                return num
        return num

    def parse_dsl(
        self,
        file_contents: str,
        outer_name: Optional[str] = None,
        instruction_def: Optional[JsonBlob] = None,
    ) -> JsonBlob:
        if instruction_def is None:
            instruction_def = {"name": None, "inputs": [], "prompts": [], "sub_instructions": []}
        lines = file_contents.split("\n")
        num_sub_instructions = 0
        line_index = 0
        while line_index < len(lines):
            line = lines[line_index].strip()
            if not line:
                pass
            elif re.match(r"^define.*\(.*\):.*$", line):
                name_start = line.find(" ") + 1
                name_end = line.find("(")
                outer_name = line[name_start:name_end]
                inputs_end = line.find(")")
                inputs = line[name_end + 1 : inputs_end].split(",")
                instruction_def["name"] = inputs[0].strip().replace('"', "")
                for instruction_input in inputs[1:]:
                    instruction_def["inputs"].append(instruction_input.strip())
            elif (
                re.match(r"^const.*=.*$", line)
                or re.match(r"^bool.*=.*$", line)
                or re.match(r"^list.*=.*$", line)
            ):
                if re.match(r"^const.*=.*$", line):
                    prompt_type = PromptType.CONST
                    suffix = (
                        "Don't include any explanations in your response.  Don't include the"
                        " question in your response."
                    )
                elif re.match(r"^bool.*=.*$", line):
                    prompt_type = PromptType.BOOL
                    suffix = "Answer 'Yes' or 'No'."
                elif re.match(r"^list.*=.*$", line):
                    prompt_type = PromptType.LIST
                    suffix = "Give your answer as a csv."
                first_space = line.find(" ")
                equals = line.find("=")
                const_name = line[first_space + 1 : equals].strip()
                last_quote = line.rfind('"')
                last_in = line.rfind(" in ")
                files = None
                if last_in and last_quote < last_in:
                    prompt_end = last_in
                    files_start = line.rfind("[")
                    files_end = line.rfind("]")
                    files = line[files_start + 1 : files_end].split(",")
                    for i, file in enumerate(files):
                        files[i] = file.strip()
                else:
                    prompt_end = -1
                prompt = (line[equals + 1 : prompt_end].strip() + " " + suffix).replace('"', "")
                instruction_def["prompts"].append(
                    {"type": prompt_type, "returns": const_name, "string": prompt}
                    | ({"files": files} if files else {})
                )
            elif re.match(r"^assert.* feedback .*$", line):
                first_space = line.find(" ")
                feedback_pos = line.find(" feedback ")
                condition = line[first_space + 1 : feedback_pos].strip()
                feedback = line[feedback_pos + 10 :].strip().replace('"', "")
                instruction_def["prompts"].append(
                    {"type": PromptType.ASSERT, "condition": condition, "feedback": feedback}
                )
            elif re.match(r"^for.* in .*$", line):
                first_space = line.find(" ")
                in_start = line.find(" in ")
                colon = line.find(":")
                variable = line[first_space + 1 : in_start]
                list_name = line[in_start + 4 : colon]
                num_sub_instructions += 1
                indent = self.get_indent(lines[line_index])
                sub_lines = []
                line_index += 1
                while line_index < len(lines):
                    if self.get_indent(lines[line_index]) <= indent:
                        line_index -= 1
                        break
                    sub_lines.append(lines[line_index])
                    line_index += 1
                sub_instruction = self.parse_dsl(
                    "\n".join(sub_lines),
                    outer_name="name",
                    instruction_def={
                        "name": instruction_def["name"],
                        "inputs": [],
                        "prompts": [],
                        "sub_instructions": [],
                    },
                )
                instruction_def["sub_instructions"].append(
                    sub_instruction["name"] | {"list": list_name, "variable": variable}
                )
            else:
                raise ValueError(f"Invalid syntax in line:\n{line}")
            line_index += 1
        assert isinstance(outer_name, str)
        return {outer_name: instruction_def}

    def parse_instructions(self, instructions: str) -> None:
        self.instructions = []
        for instruction in instructions.split("\n"):
            space_index = instruction.find(" ")
            instruction_parts = instruction[space_index + 1 :].split(",")
            instruction_number = instruction[:space_index]
            instruction_name = self.get_instruction_name(instruction_parts[0])
            this_instruction = Instruction(
                instruction_name,
                instruction_number,
                self.instruction_defs[instruction_name],
                self.code,
                self.files,
                debug_mode=self.debug_mode,
                debug_folder_path=self.debug_folder_path,
                instruction_parts=instruction_parts,
            )
            self.instructions.append(this_instruction)

    def get_instruction_name(self, name: str) -> str:
        for instruction_name in self.instruction_defs:
            if (
                self.instruction_defs[instruction_name]["name"].lower().strip()
                == name.lower().strip()
            ):
                return str(instruction_name)
        raise ValueError(f'Invalid instruction: "{name}"')

    def run_instruction(self, instruction: Instruction) -> list[str]:
        all_feedback = []
        while not instruction.done():
            queries, feedback = instruction.process_prompts()
            for query in queries:
                if self.debug_mode:
                    write_to_file(
                        f"{self.debug_folder_path}/query_{self.query_index}.txt", query[1]
                    )
                    self.query_index += 1
            all_feedback += feedback
            if not queries:
                continue
            gpt_feedback = self.gpt_adapter.get_responses(
                [adapter.GPTRequest([{"role": role.value, "content": q}]) for (role, q) in queries]
            )
            instruction.process_feedback(gpt_feedback)
        if sub_instructions := instruction.get_sub_instructions():
            for sub_instruction in sub_instructions:
                all_feedback += self.run_instruction(sub_instruction)
        return all_feedback

    def run(self) -> str:
        feedback = []
        for instruction in self.instructions:
            if instruction_feedback := self.run_instruction(instruction):
                feedback += instruction_feedback
            else:
                feedback.append(
                    f"{instruction.number} {instruction.definition['name']} is correctly"
                    " implemented."
                )
        if self.debug_mode:
            print("\n".join(feedback))
        return "\n".join(feedback)



import os
import sys
import warnings
from typing import Any

import requests
from urllib3.exceptions import InsecureRequestWarning

# Suppress InsecureRequestWarning
warnings.filterwarnings("ignore", category=InsecureRequestWarning)

JsonBlob = dict[str, Any]

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python your_script.py <code_folder>")
        sys.exit(1)

    code_folder = sys.argv[1]
    all_code = []
    for root, _, files in os.walk(code_folder):
        for file in files:
            if file.endswith(".java"):
                with open(os.path.join(root, file), "r") as f:
                    all_code.append(f.read())

    server_address = f"https://{os.getenv('SERVER_HOST')}:{os.getenv('SERVER_PORT_HTTPS')}"

    context: list[JsonBlob] = []
    while question := input("Enter question: "):
        context.append({"role": "user", "content": question})
        answer = requests.post(
            f"{server_address}/modules/SED/exercises/exercise_4/autota/ask",
            json={
                "code": "\n\n\n".join(all_code),
                "context": context,
            },
            headers={
                "Content-Type": "application/json",
            },
            timeout=5,
            verify=False,
        ).json()["answer"]
        print(answer)
        context.append({"role": "assistant", "content": answer})



from setuptools import setup, find_packages

setup(
    name='mymark',
    version='1.0.9',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'mymark=src.marker.mymark.mymark:main',
        ],
    },
    install_requires=[
        'aiohttp',
        'pytest',
        'pylint',
        'mypy',
        'marker',
    ]
)
